﻿using System;
using Bitrix.UI;
using Bitrix.IBlock.Components;
using Bitrix.IBlock;
using Bitrix;
using System.Collections.Generic;
using Bitrix.DataTypes;
using System.Web;
using System.Text;
using System.Collections.Specialized;
using Bitrix.DataLayer;
using Bitrix.Services.Text;
using Bitrix.IO;
using Bitrix.Services;
using System.IO;

using System.Collections.Generic;
using System.Security.Cryptography;
using System.IO.Compression;
using System.Globalization;
using System.Data;

namespace BXZip
{
    public class ZipStream : Stream
    {
        private static readonly byte[] TwelveBytes = new byte[12];
        private Stream stream;
        private byte[] header;
        private byte[] footer;
        private long position;
        bool lengthCalculated;
        private long length;
        private uint crc32 = 0xFFFFFFFF;
        private uint[] crcTable = new uint[256];
        private bool footerCalculated;
        private byte[] block;
        private int blockBufferFilled = 0;
        private int blockBufferWritten = 0;
        private ushort blockSize;
        private long compressedSize;
        private long uncompressedSize;
        private bool lastBlock;

        readonly int Crc32Offset1;
        readonly int Crc32Offset2;
        readonly int CompressedSizeOffset1;
        readonly int CompressedSizeOffset2;
        readonly int UncompressedSizeOffset1;
        readonly int UncompressedSizeOffset2;
        readonly int CentralDirectoryOffsetOffset;


        public ZipStream(Stream stream, string fileName, ushort blockSize)
        {
            if (stream == null)
                throw new ArgumentNullException("stream");

            if (String.IsNullOrEmpty(fileName) || fileName.Length < 1)
                throw new ArgumentNullException("fileName");

            if (blockSize == 0)
                throw new ArgumentOutOfRangeException("blockSize");

            this.blockSize = blockSize;
            block = new byte[(int)blockSize + 5];

            for (int i = 0; i < crcTable.Length; i++)
            {
                uint c = (uint)i;
                for (int j = 0; j < 8; j++)
                {
                    if ((c & 1) != 0)
                        c = 3988292384 ^ (c >> 1);
                    else
                        c >>= 1;
                }
                crcTable[i] = c;
            }


            this.stream = stream;


            /* Local file header:
                local file header signature     4 bytes  (0x04034b50)
                version needed to extract       2 bytes
                general purpose bit flag        2 bytes
                compression method              2 bytes
                last mod file time              2 bytes
                last mod file date              2 bytes
                crc-32                          4 bytes
                compressed size                 4 bytes
                uncompressed size               4 bytes
                filename length                 2 bytes
                extra field length              2 bytes

                filename (variable size)
                extra field (variable size)
            */

            string comment = "Bitrix powered";
            Encoding enc = Encoding.GetEncoding(CultureInfo.CurrentCulture.TextInfo.OEMCodePage);
            Encoding encComment = Encoding.Default;
            byte[] fileNameBytes = enc.GetBytes(fileName);
            byte[] commentBytes = encComment.GetBytes(comment);
            int fileNameLength = fileNameBytes.Length;
            int commentLength = commentBytes.Length;


            header = new byte[30 + fileNameLength];
            int offset = 0;
            Array.Copy(new byte[] { 80, 75, 3, 4, 20, 0, 0x08, 0, 0x08, 0 }, 0, header, offset, 10);
            Array.Copy(BitConverter.GetBytes(DosTime(DateTime.Now)), 0, header, offset += 10, 4); //

            Array.Copy(TwelveBytes, 0, header, offset += 4, 12); // crc-32, sizes

            Array.Copy(BitConverter.GetBytes(fileNameLength), 0, header, offset += 12, 2); //filename length 2 bytes
            Array.Copy(new byte[] { 0, 0 }, 0, header, offset += 2, 2); //extra field length  2 bytes
            Array.Copy(fileNameBytes, 0, header, offset += 2, fileNameLength); //filename


            /* Central directory's File header:
                central file header signature   4 bytes  (0x02014b50)
                version made by                 2 bytes
                version needed to extract       2 bytes
                general purpose bit flag        2 bytes
                compression method              2 bytes
                last mod file time              2 bytes
                last mod file date              2 bytes
                crc-32                          4 bytes
                compressed size                 4 bytes
                uncompressed size               4 bytes
                filename length                 2 bytes
                extra field length              2 bytes
                file comment length             2 bytes
                disk number start               2 bytes
                internal file attributes        2 bytes
                external file attributes        4 bytes
                relative offset of local header 4 bytes

                filename (variable size)
                extra field (variable size)
                file comment (variable size)
            */


            int dataAppendixSize = 16;
            int centralDirSize = 46 + fileNameLength + commentLength;
            int centralDirEndSize = 22 + commentLength;
            footer = new byte[dataAppendixSize + centralDirSize + centralDirEndSize];
            offset = 0;

            Array.Copy(BitConverter.GetBytes((uint)0x08074b50), 0, footer, offset, 4); //
            Array.Copy(TwelveBytes, 0, footer, offset += 4, 12); // crc-32, sizes
            Crc32Offset1 = offset;
            CompressedSizeOffset1 = offset + 4;
            UncompressedSizeOffset1 = offset + 8;

            Array.Copy(new byte[] { 80, 75, 1, 2, 45, 0, 20, 0, 0x08, 0, 0x08, 0 }, 0, footer, offset += 12, 12);
            Array.Copy(BitConverter.GetBytes(DosTime(DateTime.Now)), 0, footer, offset += 12, 4);
            Array.Copy(TwelveBytes, 0, footer, offset += 4, 12); // crc-32, sizes
            Crc32Offset2 = offset;
            CompressedSizeOffset2 = offset + 4;
            UncompressedSizeOffset2 = offset + 8;
            Array.Copy(BitConverter.GetBytes(fileNameLength), 0, footer, offset += 12, 2);
            Array.Copy(new byte[] { 0, 0 }, 0, footer, offset += 2, 2); // extra length
            Array.Copy(BitConverter.GetBytes(commentLength), 0, footer, offset += 2, 2); //filename length 2 bytes
            Array.Copy(new byte[] { 0, 0, 0, 0 }, 0, footer, offset += 2, 4); // disk=0, file type: binary, //Internal file attributes
            Array.Copy(BitConverter.GetBytes(0x00000020), 0, footer, offset += 4, 4); // External file attributes (normal/readable)
            Array.Copy(BitConverter.GetBytes(0), 0, footer, offset += 4, 4); // relative offset of local header

            Array.Copy(fileNameBytes, 0, footer, offset += 4, fileNameLength);
            Array.Copy(commentBytes, 0, footer, offset += fileNameLength, commentLength);


            /* End of central dir record:
                end of central dir signature    4 bytes  (0x06054b50)
                number of this disk             2 bytes
                number of the disk with the
                start of the central directory  2 bytes
                total number of entries in
                the central dir on this disk    2 bytes
                total number of entries in
                the central dir                 2 bytes
                size of the central directory   4 bytes
                offset of start of central
                directory with respect to
                the starting disk number        4 bytes
                zipfile comment length          2 bytes
                zipfile comment (variable size)
            */

            Array.Copy(new byte[] { 80, 75, 5, 6, 0, 0, 0, 0 }, 0, footer, offset += commentLength, 8);
            Array.Copy(BitConverter.GetBytes(1), 0, footer, offset += 8, 2);
            Array.Copy(BitConverter.GetBytes(1), 0, footer, offset += 2, 2);
            Array.Copy(BitConverter.GetBytes(centralDirSize), 0, footer, offset += 2, 4); //size
            Array.Copy(BitConverter.GetBytes((int)0), 0, footer, offset += 4, 4); //offset
            CentralDirectoryOffsetOffset = offset;
            Array.Copy(BitConverter.GetBytes(commentLength), 0, footer, offset += 4, 2); //filename length 2 bytes
            Array.Copy(commentBytes, 0, footer, offset += 2, commentLength);



        }

        private uint DosTime(DateTime dt)
        {
            return (uint)((dt.Second / 2) | (dt.Minute << 5) | (dt.Hour << 11) | (dt.Day << 16) | (dt.Month << 21) | ((dt.Year - 1980) << 25));
        }

        #region System.IO.Stream OVERRIDES

        public override bool CanRead
        {
            get { return true; }
        }

        public override bool CanSeek
        {
            get { return false; }
        }

        public override bool CanWrite
        {
            get { return false; }
        }

        public override void Flush()
        {

        }

        public override long Length
        {
            get
            {
                if (!lengthCalculated)
                {
                    long sl = stream.Length;
                    length = header.Length + stream.Length + footer.Length + (sl > 0 ? (sl / blockSize + 1) * 5 : 5);
                    // ((sl - 1) / blockSize + 1) is invalid, because we can't get the length of input stream
                    // and create an empty ending block when the last block size equals to buffer size
                    lengthCalculated = true;
                }
                return length;
            }
        }

        public override long Position
        {
            get
            {
                return position;
            }

            set
            {
                throw new NotSupportedException("This stream does not support seeking.");
            }
        }

        public override int Read(byte[] buffer, int offset, int count)
        {
            int written = 0;

            if (position < header.Length)
            {
                int toWrite = Math.Min(header.Length - (int)position, count);
                Array.Copy(header, position, buffer, offset, toWrite);

                offset += toWrite;
                count -= toWrite;
                position += toWrite;
                written += toWrite;
            }

            if (count > 0 && (!lastBlock || blockBufferWritten < blockBufferFilled))
            {
                while (true)
                {
                    // fill the rest of buffer
                    if (blockBufferWritten < blockBufferFilled)
                    {
                        int toWrite = Math.Min(blockBufferFilled - blockBufferWritten, count);

                        Array.Copy(block, blockBufferWritten, buffer, offset, toWrite);

                        count -= toWrite;
                        blockBufferWritten += toWrite;
                        written += toWrite;
                        position += toWrite;
                        offset += toWrite;
                    }

                    if (count == 0 || lastBlock)
                        break;

                    // fill new block
                    int read = stream.Read(block, 5, blockSize);
                    if (read < blockSize)
                        lastBlock = true;
                    blockBufferFilled = 5 + read;
                    blockBufferWritten = 0;

                    block[0] = (byte)(lastBlock ? 1 : 0);

                    ushort len = (ushort)read; // write len
                    block[1] = (byte)(len & 0x00FF);
                    block[2] = (byte)((len & 0xFF00) >> 8);

                    len = (ushort)(len ^ 0xFFFF); //complement len to 0xFFFF & write
                    block[3] = (byte)(len & 0x00FF);
                    block[4] = (byte)((len & 0xFF00) >> 8);

                    // calculate crc-32 of original file
                    for (int i = 5; i < blockBufferFilled; ++i)
                        crc32 = crcTable[(crc32 ^ block[i]) & 0xFF] ^ (crc32 >> 8);

                    compressedSize += blockBufferFilled;
                    uncompressedSize += read;
                }
            }

            if (count > 0 && position >= header.Length + stream.Length)
            {
                if (!footerCalculated)
                {
                    crc32 ^= 0xFFFFFFFF;

                    byte[] temp = BitConverter.GetBytes(crc32);
                    Array.Copy(temp, 0, footer, Crc32Offset1, 4);
                    Array.Copy(temp, 0, footer, Crc32Offset2, 4);

                    temp = BitConverter.GetBytes(compressedSize);
                    Array.Copy(temp, 0, footer, CompressedSizeOffset1, 4);
                    Array.Copy(temp, 0, footer, CompressedSizeOffset2, 4);

                    temp = BitConverter.GetBytes(uncompressedSize);
                    Array.Copy(temp, 0, footer, UncompressedSizeOffset1, 4);
                    Array.Copy(temp, 0, footer, UncompressedSizeOffset2, 4);

                    temp = BitConverter.GetBytes((int)(header.Length + compressedSize + 16));
                    Array.Copy(temp, 0, footer, CentralDirectoryOffsetOffset, 4);

                    if (!lengthCalculated)
                    {
                        length = header.Length + compressedSize + footer.Length;
                        lengthCalculated = true;
                    }

                    footerCalculated = true;
                }
                int toWrite = (int)Math.Min(Length - position, (long)count);
                Array.Copy(footer, position - (long)header.Length - compressedSize, buffer, offset, toWrite);

                position += toWrite;
                written += toWrite;
            }

            return written;
        }

        public override long Seek(long offset, SeekOrigin origin)
        {
            throw new NotSupportedException("This stream does not support seeking.");
        }

        public override void SetLength(long value)
        {
            throw new NotSupportedException("This stream does not support writing.");
        }

        public override void Write(byte[] buffer, int offset, int count)
        {
            throw new NotSupportedException("This stream does not support writing.");
        }

        #endregion
    }

    public class DataRecordStream : Stream
    {
        #region CONSTRUCTORS

        public DataRecordStream(IDataRecord record, int fieldIndex)
        {
            if (record == null)
                throw new ArgumentNullException("record");

            if ((fieldIndex < 0) || (fieldIndex >= record.FieldCount))
                throw new ArgumentOutOfRangeException("fieldIndex", fieldIndex, "Invalid field index.");

            m_record = record;
            m_field = fieldIndex;
        }

        public DataRecordStream(IDataRecord record, int fieldIndex, long length)
            : this(record, fieldIndex)
        {
            if (length < 0)
                throw new ArgumentOutOfRangeException("length", length, "The length cannot be less than zero.");

            m_length = length;
        }

        #endregion

        #region System.IO.Stream OVERRIDES

        public override bool CanRead
        {
            get { return true; }
        }

        public override bool CanSeek
        {
            get { return true; }
        }

        public override bool CanWrite
        {
            get { return false; }
        }

        public override long Length
        {
            get
            {
                if (m_length == -1)
                {
                    m_length = m_record.GetBytes(m_field, 0, null, 0, 0);
                    // We don't know the total size. We're as big as what could be read.
                    return m_length;// position;
                }
                else
                {
                    return m_length;
                }
            }
        }

        public override long Position
        {
            get { return m_position; }
            set { this.Seek(value, SeekOrigin.Begin); }
        }

        public override void Close()
        {
        }

        public override void Flush()
        {
        }

        public override int Read(byte[] buffer, int offset, int count)
        {
            long read = m_record.GetBytes(m_field, m_position, buffer, offset, count);

            m_position += read;

            if ((m_length != -1) && (m_position > m_length))
            {
                // The reported length was smaller than the actual size.
                // We update the length dynamically.
                m_length = m_position;
            }

            return unchecked((int)read);
        }

        // The default implementation of Stream.ReadByte is fine

        public override long Seek(long offset, SeekOrigin origin)
        {
            switch (origin)
            {
                case SeekOrigin.Begin:
                    m_position = offset;
                    break;

                case SeekOrigin.End:
                    if (m_length == -1)
                        throw new NotSupportedException("Unknown end-of-stream position. Cannot seek from end.");

                    // Let's avoid arithmetic exceptions
                    if (offset < 0)
                    {
                        System.Diagnostics.Debug.Assert(m_length >= 0);
                        m_position = m_length + offset;
                    }
                    else
                    {
                        m_position = m_length;
                    }

                    break;

                case SeekOrigin.Current:
                    // Blindly calling "m_position += offset" may underflow or overflow.
                    // Let's avoid arithmetic exceptions.
                    if ((offset < 0) && (-m_position > offset))
                    {
                        // Note for above: The negative of a positive value is possible.
                        //                 The negative of a negative value may fail 
                        //                 (-long.MinValue is impossible).
                        m_position = 0;
                    }
                    else if ((long.MaxValue - m_position) < offset)
                    {
                        m_position = long.MaxValue;
                    }
                    else
                    {
                        m_position += offset;
                    }

                    break;

                default:
                    throw new ArgumentException("Invalid seek origin.", "origin");
            }

            if (m_position < 0)
            {
                m_position = 0;
            }
            else if ((m_length != -1) && (m_position > m_length))
            {
                m_position = m_length;
            }

            return m_position;
        }

        public override void SetLength(long value)
        {
            throw new NotSupportedException("This stream does not support writing.");
        }

        public override void Write(byte[] buffer, int offset, int count)
        {
            throw new NotSupportedException("This stream does not support writing.");
        }

        #endregion

        #region PRIVATE FIELDS

        private IDataRecord m_record; // = null
        private int m_field; // = 0
        private long m_length = -1;

        private long m_position; // = 0

        #endregion
    }
}

public partial class NewFileTemplate : IBlockElementWebFormTemplate
{
	protected override void OnInit(EventArgs e)
	{
		base.OnInit(e);

		if (Component.IsPermissionDenied)
			Bitrix.Security.BXAuthentication.AuthenticationRequired();

		Component.CreateFieldPublicEditor += new EventHandler<IBlockElementWebFormComponent.FieldPublicEditorEventArgs>(CreateFieldPublicEditor);
		Component.CreateCustomTypePublicEditor += new EventHandler<IBlockElementWebFormComponent.CustomTypePublicEditorEventArgs>(CreateCustomTypePublicEditor);
	}

	protected override void OnLoad(EventArgs e)
	{
		base.OnLoad(e);

		BXPublicPage publicPage = Page as BXPublicPage;

		if (publicPage != null && Component.Element != null)
			publicPage.Title = "Обновить файл";
		
	}

	protected void CreateFieldPublicEditor(object component, IBlockElementWebFormComponent.FieldPublicEditorEventArgs args)
	{
		if (args.ID.ToUpper() != "SECTIONS")
			return;

		args.PublicEditor = new FileElementSections();
	}

	protected void CreateCustomTypePublicEditor(object component, IBlockElementWebFormComponent.CustomTypePublicEditorEventArgs args)
	{
		if (args.ID.ToUpper() != "FILE")
			return;

		args.PublicEditor = new FileUploadCustomType(BXCustomTypeManager.GetCustomType(args.CustomField.CustomTypeId));
	}

	public class FileElementSections : BXIBlockElementFieldPublicEditor
	{
		private List<int> fieldValues = new List<int>();

		private int iblockId;
		private int maxSectionSelect;
		private bool onlyLeafSelect;
		private bool multiple;
		private bool required;
		private string fieldTitle;
		private int textBoxSize;
		private string[] postValues;

		bool isCategorySet = true;
		bool isLangSet = true;

		private OrderedDictionary categories;
		private OrderedDictionary languages;
		private OrderedDictionary technologies;

		private class SectionTreeItem
		{
			public int Id;
			public string Name;
			public int DepthLevel;
			public bool HasChildren = false;
		}

		private OrderedDictionary GetFileSections(string sectionCode)
		{
			OrderedDictionary result = new OrderedDictionary();

			BXIBlockSectionCollection sections = BXIBlockSection.GetList(
				new BXFilter(
					new BXFilterItem(BXIBlockSection.Fields.ActiveGlobal, BXSqlFilterOperators.Equal, "Y"),
					new BXFilterItem(BXIBlockSection.Fields.Code, BXSqlFilterOperators.Equal, sectionCode),
					new BXFilterItem(BXIBlockSection.Fields.IBlock.Code, BXSqlFilterOperators.Equal, "files")
				),
				null
			);

			if (sections.Count < 1)
				return result;

			sections = BXIBlockSection.GetList(
				new BXFilter(
					new BXFilterItem(BXIBlockSection.Fields.ActiveGlobal, BXSqlFilterOperators.Equal, "Y"),
					new BXFilterItem(BXIBlockSection.Fields.SectionId, BXSqlFilterOperators.Equal, sections[0].Id)
				),
				new BXOrderBy(
					new BXOrderByPair(BXIBlockSection.Fields.LeftMargin, BXOrderByDirection.Asc)
				),
				null,
				null,
				BXTextEncoder.HtmlTextEncoder
			);

			int currentIndex = 0;
			foreach (BXIBlockSection section in sections)
			{
				SectionTreeItem item = new SectionTreeItem();
				item.Id = section.Id;
				item.DepthLevel = section.DepthLevel;
				item.Name = section.Name;
				result.Insert(currentIndex++, section.Id, item);
			}

			return result;
		}

		public override void Load(BXIBlockElement iblockElement, BXParamsBag<object> settings)
		{
			required = settings.ContainsKey("required") ? (bool)settings["required"] : true;
			fieldTitle = settings.ContainsKey("fieldTitle") ? (string)settings["fieldTitle"] : String.Empty;
			textBoxSize = settings.ContainsKey("textBoxSize") ? (int)settings["textBoxSize"] : 30;
			iblockId = settings.ContainsKey("iblockId") ? (int)settings["iblockId"] : 0;
			onlyLeafSelect = settings.ContainsKey("onlyLeafSelect") ? (bool)settings["onlyLeafSelect"] : false;
			maxSectionSelect = settings.ContainsKey("maxSectionSelect") ? (int)settings["maxSectionSelect"] : 3;
			multiple = maxSectionSelect > 1;

			categories = GetFileSections("bycategory");
			languages = GetFileSections("bylang");
			technologies = GetFileSections("bytechnology");

			if (iblockElement != null)
			{
				foreach (BXIBlockElement.BXInfoBlockElementSection section in iblockElement.Sections)
					fieldValues.Add(section.SectionId);
			}
		}

		public override string Render(string formFieldName, string uniqueID)
		{

			string fieldHeader = @"<tr field field-sections{1}""><td align=right width=30% valign=top><label class=""field-title"">{0}</label></td><td valign=top>";
			string fieldFooter = "</td></tr>";
			string dropDownStart = @"<select name=""{0}"" class=""custom-field-list"">";
			string dropDownOption = @"<option value=""{0}""{1}>{2}</option>";
			string dropDownEnd = "</select>";
            string checkbox = @"<input name=""{0}"" type=""checkbox"" value=""{1}"" id=""{2}""{3}/> <label for=""{2}"">{4}</label><BR>";

			StringBuilder result = new StringBuilder(String.Empty);

			//Categories
			result.AppendFormat(fieldHeader, @"Категория<span style=""color: red;"">*</span>", isCategorySet ? "" : " field-error");
			result.AppendFormat(dropDownStart, HttpUtility.HtmlEncode(formFieldName));
			result.AppendFormat(dropDownOption, 0, String.Empty, "(выберите категорию)");

			for (int i = 0; i < categories.Count; i++)
			{
				SectionTreeItem section = (SectionTreeItem)categories[i];

				result.AppendFormat(
					dropDownOption,
					section.Id,
					fieldValues.Contains(section.Id) ? " selected=\"selected\"" : String.Empty,
					section.Name
				);
			}

			result.Append(dropDownEnd);
			result.AppendFormat(fieldFooter);

			//Languages
			result.AppendFormat(fieldHeader, @"Язык программирования<span style=""color: red;"">*</span>", isCategorySet ? "" : " field-error");
			result.AppendFormat(dropDownStart, HttpUtility.HtmlEncode(formFieldName));
			result.AppendFormat(dropDownOption, 0, String.Empty, "(выберите язык)");

			for (int i = 0; i < languages.Count; i++)
			{
				SectionTreeItem section = (SectionTreeItem)languages[i];

				result.AppendFormat(
					dropDownOption,
					section.Id,
					fieldValues.Contains(section.Id) ? " selected=\"selected\"" : String.Empty,
					section.Name
				);
			}

			result.Append(dropDownEnd);
			result.AppendFormat(fieldFooter);

			//Technologies
			result.AppendFormat(fieldHeader, @"Технологии", "");


			for (int i = 0; i < technologies.Count; i++)
			{
				SectionTreeItem section = (SectionTreeItem)technologies[i];
				result.AppendFormat(
					checkbox,
					HttpUtility.HtmlEncode(formFieldName),
					section.Id,
					HttpUtility.HtmlEncode(formFieldName) + section.Id,
					fieldValues.Contains(section.Id) ? " checked=\"checked\"" : String.Empty,
					section.Name
				);
			}

			result.AppendFormat(fieldFooter);

			return result.ToString();
		}

		public override void Save(string formFieldName, BXIBlockElement iblockElement, BXCustomPropertyCollection properties)
		{
			if (iblockElement == null)
				return;

			BXIBlockElement.BXInfoBlockElementSectionCollection sections = iblockElement.Sections;
			sections.Clear();
			foreach (int sectionId in fieldValues)
				sections.Add(sectionId);
		}

		public override bool Validate(string formFieldName, ICollection<string> errors)
		{
			postValues = HttpContext.Current.Request.Form.GetValues(formFieldName);
			if (postValues == null || postValues.Length < 1)
			{
				fieldValues.Clear();
				errors.Add("Не указана категория файла");
				return false;
			}

			isCategorySet = false;
			isLangSet = false;

			fieldValues = new List<int>(postValues.Length);
			foreach (string value in postValues)
			{
				int sectionId;
				if (!int.TryParse(value, out sectionId) || sectionId < 1)
					continue;

				if (categories.Contains(sectionId))
				{
					isCategorySet = true;
					fieldValues.Add(sectionId);
				}
				else if (languages.Contains(sectionId))
				{
					isLangSet = true;
					fieldValues.Add(sectionId);
				}
				else if (technologies.Contains(sectionId))
				{
					fieldValues.Add(sectionId);
				}
			}

			if (!isCategorySet)
				errors.Add(String.Format("Не указана категория файла", fieldTitle));
			else if (!isLangSet)
				errors.Add(String.Format("Не указан язык программирования", fieldTitle));

			return isCategorySet && isLangSet;
		}
	}

	public class FileUploadCustomType : BXCustomTypePublicEdit
	{
		//FIELDS
		private BXCustomType type;
		private BXCustomField field;
		private BXCustomProperty property;
		private FieldEditor editor;
		private int maxSize;
		private List<string> allowedExtensions;
		private int textboxSize;

		public FileUploadCustomType(BXCustomType type)
		{
			this.type = type;
		}

		//METHODS
		private string RenderFile(BXFile file, string formName, string id, bool allowDelete, bool addPlaceholder, bool showUpload)
		{
			return
				(file != null
				? string.Format(
					allowDelete
						? string.Format(
							@"<input type=""checkbox"" name=""{0}$save"" id=""{1}"" value=""{2}"" checked=""checked"" /><label for=""{1}""> {{0}}</label><br/>",
							HttpUtility.HtmlEncode(formName),
							HttpUtility.HtmlEncode(id),
							file.Id
						)
						: @"{0}<br/>",
					string.Format(
						@"{0} {1}",
						HttpUtility.HtmlEncode(file.FileNameOriginal ?? string.Empty),
						HttpUtility.HtmlEncode(BXStringUtility.BytesToString(file.FileSize))
					)
				)
				: string.Empty)
				+ (showUpload
					? string.Format(
						@"<input type=""file"" name=""{0}"" {1}/>",
						HttpUtility.HtmlEncode(formName),
						textboxSize > 0 ? (@"size=""" + textboxSize + @""" ") : string.Empty
					)
					: string.Empty)
				+ (addPlaceholder ? @" {0}" : null);
		}
		private BXFile ValidateFile(HttpPostedFile file, ICollection<string> errors)
		{
			bool result = true;
			if (maxSize > 0 && file.ContentLength > maxSize)
			{
				result = false;
				errors.Add(HttpUtility.HtmlEncode(string.Format(
					"Размер файла {0} превышает {1}",
					file.FileName,
					BXStringUtility.BytesToString(maxSize)
				)));
			}
			if (allowedExtensions.Count != 0)
			{
				string ext = Path.GetExtension(file.FileName);
				if (ext != null)
					ext = ext.TrimStart('.').ToLowerInvariant();
				if (!allowedExtensions.Contains(ext))
				{
					result = false;
					errors.Add(HttpUtility.HtmlEncode(string.Format(
						"Тип загружаемого файла {0} не является допустимым ({1})",
						file.FileName,
						string.Join(", ", allowedExtensions.ToArray())
					)));
				}
			}
			if (!result)
				return null;

			BXFile f = null;
			string fileName = Path.GetFileName(file.FileName);
			if (!String.IsNullOrEmpty(fileName))
			{
				if (fileName.EndsWith(".zip") || fileName.EndsWith(".rar"))
					f = new BXFile(file, "files", "iblock", string.Empty);
				else
					f = new BXFile(new BXZip.ZipStream(file.InputStream, fileName, ushort.MaxValue / 2), fileName + ".zip", "files", "iblock", "");

				if (!BXSecureIO.CheckUpload(f.FileVirtualPath))
				{
					errors.Add(HttpUtility.HtmlEncode(string.Format("Недостаточно прав для загрузки файла {0}", fileName)));
					return null;
				}
			}

			return f;
		}

		public override void Init(BXCustomField field)
		{
			this.field = field;
			maxSize = field.Settings.GetInt("MaxSize");
			IEnumerable<string> ae = field.Settings.Get<IEnumerable<string>>("AllowedExtensions");
			allowedExtensions = ae != null ? new List<string>(ae) : new List<string>();
			for (int i = 0; i < allowedExtensions.Count; i++)
				allowedExtensions[i] = allowedExtensions[i].Trim().ToLowerInvariant();
			textboxSize = field.Settings.GetInt("TextBoxSize");
			editor = (FieldEditor)new SingleFieldEditor(this);
		}

		public override void Load(BXCustomProperty property)
		{
			this.property = property;
			editor.Load();
		}
		public override string Render(string formFieldName, string uniqueID)
		{
			return editor.Render(formFieldName, uniqueID);
		}
		protected override void Save(string formFieldName, BXCustomPropertyCollection storage)
		{
			editor.Save(formFieldName, storage);
		}
		protected override bool Validate(string formFieldName, ICollection<string> errors)
		{
			return editor.Validate(formFieldName, errors);
		}

		//NESTED CLASSES
		abstract class FieldEditor
		{
			protected FileUploadCustomType editor;

			public FieldEditor(FileUploadCustomType editor)
			{
				this.editor = editor;
			}

			public abstract void Load();
			public abstract string Render(string formFieldName, string uniqueID);
			public abstract void Save(string formFieldName, BXCustomPropertyCollection storage);
			public abstract bool Validate(string formFieldName, ICollection<string> errors);
		}
		sealed class SingleFieldEditor : FieldEditor
		{
			BXFile file;
			BXFile store;
			BXFile save;
			BXFile delete;

			public SingleFieldEditor(FileUploadCustomType editor)
				: base(editor)
			{

			}

			public override void Load()
			{
				if (editor.property.Value == null)
					return;

				file = BXFile.GetById((int)editor.property.Value);
			}
			public override string Render(string formFieldName, string uniqueID)
			{
				return editor.RenderFile(file, formFieldName, uniqueID, !editor.field.Mandatory, false, true);
			}
			public override void Save(string formFieldName, BXCustomPropertyCollection storage)
			{
				if (delete != null)
				{
					delete.Delete();
					delete = null;
				}

				file = store;

				if (save != null)
				{
					save.Save();

					if (file != null)
						file.Delete();
					file = save;
				}

				BXCustomProperty p = new BXCustomProperty(editor.field.Name, editor.field.Id, editor.type.DbType, false, null, true);
				if (file != null)
					p.Values.Add(file.Id);
				storage[editor.field.Name] = p;
			}
			public override bool Validate(string formFieldName, ICollection<string> errors)
			{
				HttpRequest request = HttpContext.Current.Request;

				// If the field is mandatory, then there is no checkbox
				store = file;
				if (!editor.field.Mandatory)
				{
					string save = request.Form[formFieldName + "$save"];
					int i;
					if (store != null && (!int.TryParse(save, out i) || store.Id != i))
					{
						delete = store;
						store = null;
					}
				}

				HttpPostedFile upload = request.Files[formFieldName];
				if (upload != null && upload.ContentLength != 0)
				{
					BXFile uploaded = editor.ValidateFile(upload, errors);
					if (uploaded == null)
						return false;

					save = uploaded;
				}

				if (editor.field.Mandatory && store == null && save == null)
				{
					errors.Add(HttpUtility.HtmlEncode(string.Format("Не заполнено поле \"{0}\"", editor.field.TextEncoder.Decode(editor.field.EditFormLabel))));
					return false;
				}

				return true;
			}
		}
	}
}